<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


//  calc_average()      - calculates min, max and average. Only for RRA: AVERAGE
//  @arg $pre_data      - contains an array witch the prepared datas to calculate with
//  @arg $parameters    - contains an array of parameters/variables
//  @returns		- an array with min, max and average value for every RRD of report's rrdlist

function calc_average(&$pre_data, &$parameters) {
    if(isset($parameters['bandwidth'])) {
	$bandwidth 	= $parameters['bandwidth'];
    }
    
    $multiplier 	= $parameters['multiplier'];
    
    for($i=0; $i < $parameters['rrd_ds_cnt']; $i++) {
	
	$average_average	= array_sum($pre_data[$i])/count($pre_data[$i]) * $multiplier;
	$average_max 		= max($pre_data[$i]) * $multiplier;
	$average_min 		= min($pre_data[$i]) * $multiplier;
	
	// Save results
	if(isset($bandwidth)) {
    	    $results[$i]['BW [Bits/s]'] = $bandwidth;
	    $results[$i]['AVG [Bits/s]']= $average_average;
	    $results[$i]['MAX [Bits/s]']= $average_max;
	    $results[$i]['MIN [Bits/s]']= $average_min;
	}else {
	    $results[$i]['AVG [%]']	= $average_average;
	    $results[$i]['MAX [%]']	= $average_max;
	    $results[$i]['MIN [%]']	= $average_min;	    
	}
    }
    
    return $results;
}



//  calc_max()          - calculates min, max and average. Only for RRA: MAX
//  @arg $pre_data      - contains an array witch the prepared datas to calculate with
//  @arg $parameters    - contains an array of parameters/variables
//  @returns			- an array with min, max and average value for every RRD of report's rrdlist

function calc_max(&$pre_data, &$parameters) {
    if(isset($parameters['bandwidth'])) {
	$bandwidth 	= $parameters['bandwidth'];
    }

    $multiplier 	= $parameters['multiplier'];   

    for($i=0; $i < $parameters['rrd_ds_cnt']; $i++) {
	$max_average		= array_sum($pre_data[$i])/count($pre_data[$i]) * $multiplier;
	$max_max 		= max($pre_data[$i]) * $multiplier;
	$max_min 		= min($pre_data[$i]) * $multiplier;
	
	// Save results
	if(isset($bandwidth)) {
    	    $results[$i]['BW [Bits/s]'] = $bandwidth;	    
	    $results[$i]['AVG [Bits/s]']= $max_average;
	    $results[$i]['MAX [Bits/s]']= $max_max;
	    $results[$i]['MIN [Bits/s]']= $max_min;
	}else {
	    $results[$i]['AVG [%]']	= $max_average;
	    $results[$i]['MAX [%]']	= $max_max;
	    $results[$i]['MIN [%]']	= $max_min;	    
	}
    }
    
    return $results;
}    



//  calc_min()          - calculates min, max and average. Only for RRA: MIN
//  @arg $pre_data      - contains an array witch the prepared datas to calculate with
//  @arg $parameters    - contains an array of parameters/variables
//  @returns		- an array with min, max and average value for every RRD of report's rrdlist

function calc_min(&$pre_data, &$parameters) {
    if(isset($parameters['bandwidth'])) {
	$bandwidth 	= $parameters['bandwidth'];
    }

    $multiplier 	= $parameters['multiplier'];
   
    for($i=0; $i < $parameters['rrd_ds_cnt']; $i++) {
	$min_average		= array_sum($pre_data[$i])/count($pre_data[$i]) * $multiplier;
	$min_max 		= max($pre_data[$i]) * $multiplier;
	$min_min 		= min($pre_data[$i]) * $multiplier;
	
	// Save results
	if(isset($bandwidth)) {
    	    $results[$i]['BW [Bits/s]'] = $bandwidth;
	    $results[$i]['AVG [Bits/s]']= $min_average;
	    $results[$i]['MAX [Bits/s]']= $min_max;
	    $results[$i]['MIN [Bits/s]']= $min_min;
	}else {
	    $results[$i]['AVG [%]']	= $min_average;
	    $results[$i]['MAX [%]']	= $min_max;
	    $results[$i]['MIN [%]']	= $min_min;	    
	}
    }
    
    return $results;
}



//  calc_threshold()    - calculation in relation to defined threshold factor
//  @arg $pre_data      - contains an array witch the prepared datas to calculate with
//  @arg $parameters    - contains an array of parameters/variables
//  @returns		- an array with a lot of results

function calc_threshold(&$pre_data, &$parameters) {
    if(!isset($parameters['bandwidth'])) {
	$bandwidth 	= 100; //e.g. for CPU
    }else {
	$bandwidth 	= $parameters['bandwidth'];
    }

    $multiplier		= $parameters['multiplier'];    
    $threshold_factor 	= $parameters['variables']['threshold']/100;
    
    for($i=0; $i < $parameters['rrd_ds_cnt']; $i++) {        
	// Set preconditions
        $x     	      	= 0;
	$sum   	      	= 0;
	$over_threshold = 0;
	$under_threshold= 0;
	$step_duration	= $parameters['rrd_step'];
	$steps 	      	= count($pre_data[$i]);
	$threshold	= $bandwidth * $threshold_factor;
	$capacity 	= $bandwidth * $steps * $step_duration;
    
	
	// Analyse every measuring point (mp)
        foreach($pre_data[$i] as $value) {
	    if($value != 0) {
    		$value *= $multiplier;
		$value -= $threshold;
		$value *= $step_duration;

		switch($value) {
		    case $value > 0 :
			$over_threshold  += $value;
			$x++;
			break;
		    case $value < 0 :
			$under_threshold += -$value;
			break;
		}
	    }
	}
	
	//Calculate the transmitted pack of data
	$amount_of_data = array_sum($pre_data[$i]) * $multiplier * $step_duration;
	    
      	if($capacity != 0) {
	    // Calculate average free capacity ("free breath")
	    	    
	    $cap_over_threshold		= ($capacity * (1 - $threshold_factor));
    	    $free_cap_average		= round((100 *(($cap_over_threshold - $over_threshold) / $cap_over_threshold)), 2);
	
	    // Calculate the capacity utilization
	    $cap_utilization		= round((100 *  $amount_of_data / $capacity), 2);
	}else {
	    $free_cap_average		= '';
	    $cap_utilization		= '';
	}

	// Calculate mp's over threshold in relation to number of steps
	$duration_over_threshold= ($x / $steps)*100;

	// Save results
	if(isset($parameters['bandwidth'])) {
	    // Results for traffic
	    $results[$i]['BW [Bits/s]']	= $bandwidth;
	    $results[$i]['TH [Bits/s]'] = $bandwidth * $threshold_factor;
	    $results[$i]['CAP [Bits]']  = $capacity;
	    $results[$i]['AOD [Bits]']	= $amount_of_data;
	    $results[$i]['OT [Bits]']	= $over_threshold;
	}
	
	// Result for traffic, CPU ...
	$results[$i]['FCAP [%]']	= $free_cap_average;
	$results[$i]['UTIL [%]']	= $cap_utilization;
	$results[$i]['DOT [%]']		= $duration_over_threshold;
    }
    
    return $results;
}
?>