<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


//@report_data 		- Contains all input data
//@export_settings 	- Contains the globally or user specific export settings
function export_to_csv($export_data, $export_settings) {

include_once("./plugins/reportit/lib_int/const_view.php");

    global $config, $plugin_architecture, $abbreviations;
    
    //Set some arrays
    $csv_column_seperator 	= array(",", ";", "\t", " ");
    $csv_decimal_seperator	= array(",", ".");
    $csv_date_format		= array("m");

    //Check if user settings does not exist
    if (!isset($export_settings['reportit_csv_column_s'])) {
	$export_settings['reportit_csv_column_s'] = '1';
    }
    if (!isset($export_settings['reportit_csv_decimal_s'])) {
	$export_settings['reportit_csv_decimal_s'] = '0';
    }    
    if (!isset($export_settings['reportit_exp_header'])) {
	$export_settings['reportit_exp_header'] = '';
    }
    
    //Set some variables
    $report_data		= $export_data['report_data']; 
    $report_results		= $export_data['report_results'];
    $id   			= $export_data['report_data']['id'];    
    $path                       = $config['base_path'] . '/plugins/reportit/exports/csv';
    
    $csv_seperator		= $csv_column_seperator[$export_settings['reportit_csv_column_s']];  
    $dec_seperator		= $csv_decimal_seperator[$export_settings['reportit_csv_decimal_s']];

    $reportit_version		= reportit_version();
    
    $info_line			= '';    
    $legend 			= "# Legend:\r\n";    
    $tab_header 		= '';    


    //Extract result description
    $rs_description		= explode("|", $report_data['result_description']);   
        
    
    //Build filename and it's path
    $filename 			= str_replace("<report_id>", "$id", $export_settings['reportit_filename']) . ".csv";
    $cmp_path			= "$path/$filename";
    
    
    //Clear the old CSV-file 
    if (file_exists($cmp_path)) {
    	unlink($cmp_path);
    }


    //Create an new handle
    $filehandle 		= fopen($cmp_path, 'a');
    
    
    //Transform and save CSV header
    $export_settings['reportit_exp_header'] = str_replace("<cacti_version>", "Cacti: " . $config['cacti_version'], $export_settings['reportit_exp_header']);
    $export_settings['reportit_exp_header'] = str_replace("<report_version>", "Report-Plugin: " . $reportit_version['version'], $export_settings['reportit_exp_header']);
    fwrite($filehandle, "{$export_settings['reportit_exp_header']}\r\n\r\n");

    
    //Build and save information line
    $csv_header = array('Description' 		=> "{$report_data['description']}",
			'Owner'			=> "{$report_data['owner']}",
			'Template'		=> "{$report_data['template_name']}",
			'Type'			=> "{$report_data['type']['type']}",
			'Calculation function'	=> "{$report_data['type']['description']}",
			'Start'			=> "{$report_data['start_date']}",
			'End'			=> "{$report_data['end_date']}",    
			'Last Run'		=> "{$report_data['last_run']}"
		        );
    foreach($csv_header as $key => $value) {
	$info_line = $info_line . "# $key: $value\r\n";
    }    

    fwrite($filehandle, "$info_line\r\n");     


    //Build an legend for our abbreviations
    foreach($rs_description as $value) {
	if(array_key_exists($value, $abbreviations)) {
		    $legend = $legend . "# $value: {$abbreviations[$value]}\r\n";	
	}
    }
    
    fwrite($filehandle, "$legend\r\n");  


    //Transform and save table header
    foreach($report_results[0] as $key => $value) {
	$tab_header = $tab_header . '"'.$key.'"' . $csv_seperator;
    }
    
    fwrite($filehandle, "$tab_header\r\n");    
    

    //Transform and save table values
    foreach($report_results as $result) {
	$row = '';
	foreach($result as $key => $value) {
	    //Check $value format
	    if (strpos($key, '__')){
		//We should round here
		$value = round($value, 2);
		//Format $value with defined decimal seperator
		$value = str_replace(".", $dec_seperator, $value);
	    }
	    $row = $row . '"'.$value.'"' . $csv_seperator;
	}
        fwrite($filehandle, "$row\r\n");
    }


    //Close handle
    fclose($filehandle);
    
    return $filename;
}

?>