<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


function create_result_table($report_id) {
    // Create the sql syntax
    $sql = "CREATE TABLE reportit_results_" . $report_id . " ( 
	    id 			INT 		PRIMARY KEY 
	   )TYPE=MyISAM";    
    db_execute($sql);
    // Copy all actual ids from rrdlist
    $sql = "INSERT INTO 	reportit_results_" . $report_id . 
	   " SELECT id FROM 	reportit_rrdlist_" . $report_id;
    db_execute($sql);
}


function &get_report_definitions($report_id) {
    // Fetch all standard report definitons for reportit
    $sql 	= "SELECT * FROM reportit_reports WHERE id = $report_id";
    $report_definitions	= db_fetch_assoc($sql);

    // Fetch all RRD definitions 
    $sql 	= "SELECT id as local_data_id, start_day, end_day, start_time, end_time, timezone
		   FROM reportit_rrdlist_$report_id";
    $rrd_list	= db_fetch_assoc($sql);

    // Fetch all definitions for calculation function
    $sql = "SELECT 	c.calc_function, 
			c.included_rra 
	    FROM	reportit_reports 				AS a 
	    LEFT JOIN	reportit_templates 				AS b 
	    ON		b.id 			= a.template_id 
	    LEFT JOIN	reportit_functions 				AS c 
	    ON		c.id			= b.function_id 
	    WHERE	a.id  			= $report_id"; 
    $rra_type = db_fetch_row($sql);

    // Fetch all current bandwith parameters 
    $sql = "SELECT 	c.field_value, b.id 
	    FROM 	reportit_rrdlist_$report_id 			AS a 
	    LEFT JOIN	data_local					AS b 
	    ON		b.id			= a.id 
	    LEFT JOIN	host_snmp_cache					AS c 
	    ON		c.host_id 		= b.host_id  		AND 
			c.snmp_index 		= b.snmp_index  	AND 
			c.snmp_query_id 	= b.snmp_query_id 
	    WHERE 
			c.field_name 		= 'ifspeed' 
	    ORDER BY 	b.id 						ASC";
    $bandwidths = db_fetch_assoc($sql);
    
    // Fetch all template variables
    $sql = "SELECT 	a.field_name, 
			a.value
	    FROM	reportit_variables 				AS a
	    RIGHT JOIN	reportit_reports 				AS b
	    ON		a.template_id 		= b.template_id
	    WHERE	b.id			= $report_id";
    $variables = db_fetch_assoc($sql);

    // Fetch multiplier (set by template)
    $sql = "SELECT 	c.multiplier
	    FROM 	reportit_reports 				AS a
	    JOIN	reportit_templates 				AS b
	    ON 		a.template_id 		= b.id
	    JOIN	reportit_types 					AS c
	    ON		b.type_id 		= c.id
	    WHERE	a.id 			= $report_id";
    $multiplier = db_fetch_row($sql);

    // Transform $variables and put results into $var_list array
    if(count($variables)>0) {
	foreach($variables as $key => $value) {
	    $var_list[$variables[$key]['field_name']] = $variables[$key]['value'];
	}
    }

    // Update bandwith parameters of rrd_list if exists. (e.g. CPU has no bandwidth parameter)
    if(count($bandwidths)>0) {
	foreach($rrd_list as $key_a => $value_a) {
	    $local_id = $rrd_list[$key_a]['local_data_id'];
    	    foreach($bandwidths as $key_b => $value_b) {
		if($bandwidths[$key_b]['id'] == $local_id) {
		    break;
		}
	    }
	    $rrd_list[$key_a]['bandwidth'] = $bandwidths[$key_b]['field_value'];
	}
    }
  
    //Construct the return-array "report_definitions"
    $report_definitions[] = $rrd_list;
    $report_definitions[] = $rra_type;
    $report_definitions[] = $var_list;
    $report_definitions[] = $multiplier;
    
    return $report_definitions;
}


function get_runtime($start_time, $end_time) {
    // Calculate the time a script needs for execution
    list($startmsec, $startsec) = explode(" ",$start_time);
    list($endmsec, $endsec) 	= explode(" ",$end_time);
    $runtime 			= round(($endmsec+$endsec) - ($startmsec+$startsec), 1);

    return $runtime;
}


function day_to_number($day) {
    switch($day)
    {
	case "Monday":
	    return 1; break;
	case "Tuesday":
	    return 2; break;
	case "Wednesday":
	    return 3; break;
	case "Thursday":
	    return 4; break;
	case "Friday":
	    return 5; break;
	case "Saturday":
	    return 6; break;
	case "Sunday":
	    return 7; break;
    }
}


function &get_type_of_request($startday, $endday, $f_sp, $l_sp, $shift_duration, $rrd_sp, $rrd_ep, $rrd_step, $rrd_ds_cnt) {
    /*-----------------------------------------------------------------------------------------------------------
    	Calculate all included weekdays 
    	For a great report duration it's more efficient to use time vectors instead of "get weekday function"
    
    	Variables:	$wdays 		=> includes all ids of weekdays which are include
    			$dis 		=> means the distance vector between for all included days
    			$off		=> means the distance vector between for all not included days	
    			$rrd_ad_data 	=> Return value of this function. An array that includes all important
    					   informations to grep the adapted data out of rrd_data array. 
    -----------------------------------------------------------------------------------------------------------*/

    //----- Calculate all included weekdays -----
    Switch ($startday) {
	case $startday == $endday:				//e.g. 'Monday till Monday => includes only Monday!'
	    if ($startday == 7) {
		$wdays = array(0);
	    }else {
		$wdays = array($startday);
	    }
	    $dis = 0;
	    $off = 7;
	    break;
	    
	case $startday <  $endday:    				//e.g. 'Monday till Friday => includes Mo, Tu, Wed, Thu and Fr
	    $dis  = $endday - $startday;
	    $off  = 7 - $dis;

	    for($startday; $startday <= $endday; $startday++) {
		$wdays[]= $startday;
	    }
	    
	    if($endday == 7) {$wdays[]= 0;}
	    break;
	     
	case $startday >  $endday:				//e.g. 'Friday till Monday => includes Fr, Sat, Sun, Mo'
	    $dis = 7 - $startday + $endday;
	    $off = 7 - $dis;

	    for($startday; $startday <= 7; $startday++) {
		if ($startday == 7) {
		    $wdays[]= 0;
		}else {
		    $wdays[]= $startday;
		}
	    }

	    for($offset = 1; $offset <= $endday; $offset++) {
		$wdays[]= $offset;
	    }
	    break;
    }
    if ($endday == 7) {$endday = 0;}
    //-------------------------------------------
    
    
    //----- Calculate all starting points which will be included in report duration -----
    // Using "classic" way until first endday is found.
    
    // Set preconditions
    $breakup 	= false;
    $date 	= getdate($f_sp);
    $fdis	= 0;

    for($f_sp; $f_sp <= $l_sp AND $breakup == false; $f_sp+=86400, $date=getdate($f_sp)) {
	if(in_array($date['wday'],$wdays)){
	    $all_sp[]= $f_sp;
	    $fdis++ ;
	}
	if($date['wday'] == $endday){$breakup = true;}
    }

    $f_sp -=86400;
    //-----------------------------------------------------------------------------------


    //----- Calculate all starting points which will be included in report duration -----
    //Using time vectors until end is reached.

    //Set preconditions
    $breakup	= false;
    $offs	= 0;
    $ldis	= 0;


    while( $breakup == false ) {

	//Set startingpoint to the next duration
        $f_sp += ($off * 86400);

	//Reset last distance vector
	$ldis = 0;

	//Count number of offsets
	$offs++;

	//Start searching number of important timestamps
	    for($f_sp, $i=$dis; $f_sp <= $l_sp AND $i>=0; $f_sp+=86400, $i--) {
		$all_sp[]= $f_sp;
		$ldis++;
	    }
	//Timestamp correction:
	$f_sp -=86400;

        if($f_sp >= $l_sp){
	    $breakup = true;
	    if($ldis == 0) {
		$offs--;
	    }
	}
    }
    //-----------------------------------------------------------------------------------


    //----- ERROR CHECK (1) -----
    //Check wether a valid startpoint has been found
    if(!isset($all_sp)) {
	return false;
    }
    //---------------------------

  
    //----- Calculate all index numbers for start points -----

    //Set preconditions
    $number_of_sps = count($all_sp);
    $steps_of_day  = 86400/$rrd_step;    
    $rrd_start_point = $rrd_sp;
    $index = $steps_of_day;

    for( $x = 0; $x < $number_of_sps; $x++) {	    
	for( $y = $rrd_start_point; $y < $all_sp[$x]; $y+=86400) {
	    $index+=$steps_of_day;
	}
	//Timestamp correction
	$rrd_start_point = $y - 86400;
	//Index correction
	$index-=$steps_of_day;
	$rrd_ad_data['index'][] = $index;
    }

    //--------------------------------------------------------


    //----- Calculate number of rrd_steps for enclosing shifttime -----

    //Set preconditions 
    $next	= $all_sp[0] + $shift_duration;
    $f_end	= 0;

    for($i=$rrd_sp; $i < $next; $i+=$rrd_step) {
	    $f_end++;
    }

    $rrd_ad_data['steps'] = $f_end-$rrd_ad_data['index'][0];
    //-----------------------------------------------------------------

    //----- Finish -----
    return $rrd_ad_data;
}


function &get_prepared_data(&$rrd_data, &$rrd_ad_data, $rrd_ds_cnt, $rrd_info, $corr_factor_start, $corr_factor_end) {
    for($i = 0; $i<$rrd_ds_cnt; $i++) {
	
	//Number of indexes
	$n_of_indexes = count($rrd_ad_data['index']);
	
	for($j = 0; $j<$n_of_indexes; $j++) {
	    //Set index number
	    $number = $rrd_ad_data['index'][$j]*$rrd_ds_cnt+$i;
	    $data[$i][$number] = $rrd_data[$number];
	}
	
	//Data correction (start), if needfully (TYPE:COUNTER)
	//Without using call by reference, because I've used PHP4!
	if($rrd_info['ds_type'][$i] == 2) {
	    foreach($data[$i] as $key => $value) {
		if(!is_nan($value)) {$data[$i][$key] = $value*$corr_factor_start;}
		
	    }
	}
	
	//Fetch rest of data
	foreach($data[$i] as $key => $value) {
	    for($k = 1; $k <= $rrd_ad_data['steps']; $k++) {
		$number = $key+$k*$rrd_ds_cnt;
		$data[$i][$number] = $rrd_data[$number];
	    } 
	    
	    //Correct last value (end), if needfully (Type:COUNTER)
	    if(($rrd_info['ds_type'][$i] == 2) && (is_nan($data[$i][$number]))) {
		$data[$i][$number] *=$corr_factor_end;
	    }
	}

	//Remove all NAN's
	foreach($data[$i] as $key => $value) {
	    if(is_nan($value)) {
	    unset($data[$i][$key]);}
	}
    }
    return $data;
}



function strtofloat(&$value) {
    @settype($value, float);
}


function transform(& $data, & $rrd_data) {

    //Check operating system
    if(strpos(PHP_OS, 'WIN')) {
	$eol = "\r\n";
    }else{
	$eol = "\n";
    }

    //Transform into the 'normal' form:
    $ds_names 		    = substr($data, 0, strpos($data, $eol));
    $data		    = substr($data, strpos($data, $eol)); 

    preg_match_all('/\S+/', $ds_names, $rrd_data);
    $rrd_data['ds_namv']    = array_shift($rrd_data);	
    $rrd_data['ds_cnt']     = count($rrd_data['ds_namv']);

    preg_match_all('/\S+/', $data, $data);
    $zahl		    = count($data[0]);
    $last_timestamp 	    = $zahl - $rrd_data['ds_cnt'] - 1; 	
    $rrd_data['start']      = substr($data[0][0],0,-1);
    $rrd_data['end']        = substr($data[0][$last_timestamp], 0, -1);

    $b 			    = $rrd_data['ds_cnt'];
    $rrd_data['step']	    = $data[0][$b+1] - $rrd_data['start'];
    $rrd_data['start']     -= $rrd_data['step'];
    
    $a	   		    = 0;

    //Delete all timestamps
    while($a<$zahl) {
        unset($data[0][$a]);
        $a+=$b+1;
    }

    $a-=$b+1;

    $rrd_data['data'] 	= array_values($data[0]);
    array_walk($rrd_data['data'], 'strtofloat');
    
    unset($data);  
}

?>
