<?php
/*
 +----------------------------------------------------------------------+
 | Copyright (C) 2006 Andreas Braun                                   	|
 |									|
 | This program is free software. You can redistribute it		|
 | and/or modify it under the terms of the GNU General Public License	|
 | as published by the Free Software Foundation; either version 2       |
 | of the License, or (at your option) any later version.               |
 |                                                                      |
 | This program is distributed in the hope that it will be useful, but  |
 | WITHOUT ANY WARRANTY; without even the implied warranty of     	|
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        |
 | GNU General Public License for more details.     			|
 |                    							|
 +----------------------------------------------------------------------+
*/


//----- Define some variables -----
$PATH_RID_LOG  	= "<a href='./plugins/reportit/cc_reports.php?action=report_edit&id=<RID>'><RID></a>";
$PATH_DID_LOG  	= "<a href='./plugins/reportit/cc_rrdlist.php?action=rrdlist_edit&id=<DID>&report_id=<RID>'><DID></a>";
$PATH_RID_VIEW 	= "<a href='cc_reports.php?action=report_edit&id=<RID>'><RID></a>";
$PATH_DID_VIEW 	= "<a href='cc_rrdlist.php?action=rrdlist_edit&id=<DID>&report_id=<RID>'><DID></a>";

$run_return	= array();
$run_freq 	= '';
$run_verb	= FALSE;    
$run_scheduled  = FALSE;
$run_search	= array('<NOTICE>','<RID>','<DID>');

$socket_handle	= '';

//----- Running from command line ? -----
if($_SERVER['PHP_SELF'] == $_SERVER['argv']['0']) {
    
    $path = dirname(__FILE__);
    chdir($path); 
    chdir('../../');     
    include('./include/config.php');
    include_once('./plugins/reportit/setup.php');    
    include_once('./plugins/reportit/lib_int/funct_shared.php');
    include_once('./plugins/reportit/lib_int/const_runtime.php');
    include_once('./plugins/reportit/lib_int/funct_calculate.php');   
    include_once('./plugins/reportit/lib_int/funct_runtime.php');
    
    $run_version 	= reportit_version();
    $run_scheduled	= TRUE;

    if(($_SERVER['argc'] > '3' || $_SERVER['argc'] < '2')) help();
        
    foreach($_SERVER['argv'] as $option) {
	switch($option) {
            case "-d":
                $run_freq = 'daily';
                break;
            case "-w":
                $run_freq = 'weekly';
                break;
            case "-m":
                $run_freq = 'monthly';
                break; 
            case "-q":
                $run_freq = 'quarterly';
                break;
            case "-y":
                $run_freq = 'yearly';
                break;
            case "-v":
                $run_verb = TRUE;
                break;
	}
    }
    
    if($run_freq == '') help();
    run($run_freq);
}


function help() {
    global $run_version;
    
    echo "\n-----------------------------------------------------\n";
    echo " Scheduled Reporting - Cacti Plugin \"reportit\" V{$run_version['version']}\n\n";
    echo " Usage: runtime.php [OPTION]\n";
    echo "  e.g.: runtime.php -d -v\n\n";
    echo "  -d    :daily            -v   :verbose\n";
    echo "  -w    :weekly\n";
    echo "  -m    :monthly\n";
    echo "  -q    :quarterly\n";
    echo "  -y    :yearly\n\n";
    echo " Report bugs to <{$run_version['email']}>\n";
    echo "-----------------------------------------------------\n\n";
    exit;
}


function run($frequency) {
    global $run_verb;    

    //Check RRDtool connection:
    
    if(!chk_cnn_rrdtool()) exit;
    
    $start	= microtime();
    $sql 	= "SELECT id FROM reportit_reports WHERE scheduled = '1' AND frequency = '$frequency'";
    $reports 	= db_fetch_assoc($sql);
    $number	= count($reports);

    foreach($reports as $id) {
	$report_id = $id['id'];
	runtime($report_id);
    }
    
    $end	= microtime();
    $time 	= get_runtime($start, $end);

    cacti_log( "REPORTIT STATS: Frequency:$frequency Time:$time Reports:$number", $run_verb, 'PLUGIN');    
    exit;
}



function run_error($code, $RID = 0, $DID = 0, $notice='') {
    global $run_verb, $run_scheduled, $run_return, $run_search, $runtime_messages,
	   $PATH_RID_LOG, $PATH_DID_LOG, $PATH_RID_VIEW, $PATH_DID_VIEW;
    
    $run_output  	= '';
    $run_logging 	= '';

    $run_repl_log  	= array( $notice, $PATH_RID_LOG,  $PATH_DID_LOG);
    $run_repl_view 	= array( $notice, $PATH_RID_VIEW, $PATH_DID_VIEW);    
    $run_repl_fin	= array( $notice, $RID, $DID);

    if($run_scheduled) {
	$run_logging 	= str_replace($run_search, $run_repl_log, $runtime_messages[$code]);
	$run_logging 	= str_replace($run_search, $run_repl_fin, $run_logging);
	cacti_log($run_logging, false, 'PLUGIN');
	if($run_verb) {
	    $date 	= gmdate("m/d/Y h:i:s A");
	    $output 	= "$date - " . str_replace($run_search, $run_repl_fin, $runtime_messages[$code]) . "\n";
	    echo $output;
	}
    }else {
	$run_output  	= str_replace($run_search, $run_repl_view, $runtime_messages[$code]);
	$run_output  	= str_replace($run_search, $run_repl_fin, $run_output);
	$run_return[]	= $run_output;
    }
}


function chk_cnn_rrdtool() {
    global $rrdtool_api;

    $rrdtool_api  = read_config_option('reportit_API');
    switch($rrdtool_api) {
	case '0':
	    //Check PHP bindings
	    return check_rrdtool_bindings();
	    break;

	case '2':
	    //Check RRDtool Server
	    return check_rrdtool_server();		    
	    break;
	    
	default:
	    //RRDtool Cacti means the core function rrdtool_execute
	    //We have only to include the lib.
	    include_once("./lib/rrd.php");		    
	    return true;
	    break; 
    }
}


function check_rrdtool_bindings() {
    global $run_scheduled;
    
    $loaded_extensions 	= get_loaded_extensions();
    if(in_array('rrdtool', $loaded_extensions)) {
	return true;
    }else {
	if($run_scheduled) {
	    cacti_log( 'REPORTIT ERROR: PHP modul for RRDtool is not available.', true, 'PLUGIN');
	}else {
	    run_error(1);
	}
    }
    return false;
}


function check_rrdtool_server() {
    global $socket_handle, $run_scheduled;
    
    $run_RRDId		= read_config_option('reportit_RRDID');
    $run_RRDPort	= read_config_option('reportit_RRDPort'); 
    $socket_handle	= socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
    $socket_connection	= @socket_connect($socket_handle, $run_RRDId, $run_RRDPort);


    if(!$socket_connection) {
	if($run_scheduled) {
	    cacti_log( 'REPORTIT ERROR: Unable to connect to RRDtool server.', true, 'PLUGIN');
	}else {
	    run_error(9);
	}
    }
    return $socket_connection;
}


function disc_rrdtool_server() {

    global $socket_handle;    
    socket_close($socket_handle);
    return true;
}


// runtime()		- is the core function of this plugin, that has to handle
//			  a lot subfunctions and error checks for creating a report
// @arg $report_id	- the identifier of a report template we want to create a report for
// @returns		- an associative array that contains errors, notifications and the runtime
function runtime($report_id) {
    global $timezones, $data_source_types, $run_scheduled, $run_return, $rrdtool_api, $socket_handle;

    if(!$run_scheduled) {
	if(!chk_cnn_rrdtool()) return $run_return;
	ini_set("max_execution_time", read_config_option("reportit_met"));
    }
  
    //----- Define variables for Datasource and RRA descriptions -----
    $ds_description 	= '';
    $result_description = '';

    //----- Define variable for monitoring a valid process -----
    $valid_report 	= false;

    //----- Make a note of our startpoint -----
    $runtime_sp 	= microtime();

    //----- Reset report -----
    reset_report($report_id);

    //----- Fetch all necessary data for building a report -----
    $report_definitions	= &get_report_definitions($report_id);

    //----- Define variable for dynamic time frame -----
    $dynamic 		= $report_definitions[0]["dynamic"];

    //----- Update start and enddate by using presets -----
    if($dynamic) {
	$dates 		= get_timespan($report_definitions[0]["preset_timespan"], $report_definitions[0]["present"]);
	$report_definitions[0]["start_date"] 	= $dates['start_date'];
	$report_definitions[0]["end_date"] 	= $dates['end_date'];	
    }
    
    //----- Get number of RRD datasources -----
    $number_of_rrds = count($report_definitions[1]);


    //----- ERROR CHECK (2) -----
    //Check number of defined RRDs
    if(!$number_of_rrds > 0) {
	run_error(2,$report_id);	
	return $run_return;
    }
    //---------------------------


    //----- Prepare result table -----
    // First destroy old result table if exists
    db_execute("DROP TABLE IF EXISTS reportit_results_$report_id");

    // Create new table for saving our results
    create_result_table($report_id);

    // Set variable to control creation of new columns
    $create_result_table_columns = false;
    //--------------------------------


    //----- Report Settings -----
    $s_date	  = $report_definitions[0]['start_date'];
    $e_date	  = $report_definitions[0]['end_date'];

    //----- Get number of data source types -----
    $rrd_tp_cnt	  = count($data_source_types);

    //----- Start analysing each RRD -----
    for($i = 0; $i < $number_of_rrds; $i++) {

    	//----- Interface Settings -----
        $local_data_id	= $report_definitions[1][$i]['local_data_id'];

	If(isset($report_definitions[1][$i]['bandwidth'])) {
	    $bandwidth = $report_definitions[1][$i]['bandwidth'];
	}

    	$s_time   		= $report_definitions[1][$i]['start_time'];
	$e_time	 		= $report_definitions[1][$i]['end_time'];
        $timezone  		= $report_definitions[1][$i]['timezone'];

        $rra_type	 	= $report_definitions[2]['included_rra'];
	$calc_function	  	= $report_definitions[2]['calc_function'];

	//----- Convert weekdays -----
	$shift_startday		= day_to_number($report_definitions[1][$i]['start_day']);
	$shift_endday		= day_to_number($report_definitions[1][$i]['end_day']);

	//----- Get the local path of the RRD-file that belongs to the local_data_id -----
	$data_source_path 	= get_data_source_path($local_data_id, true);

        //----- Participate reporting times and get offset to GMT -----
	list($s_hour, $s_min)	= explode(":",$s_time);
        list($e_hour, $e_min)	= explode(":",$e_time);
	$offset_hour	      	= $timezones[$timezone]['hour'];
        $offset_min	      	= $timezones[$timezone]['min'];

	//----- Participate reporting start- and enddate -----
        list($s_year, $s_month, $s_day) = explode("-",$s_date);
	list($e_year, $e_month, $e_day) = explode("-",$e_date);

        //----- Calculate correct GMT timestamps -----
	$f_sp   = gmmktime($s_hour-$offset_hour,$s_min-$offset_min,0,$s_month,$s_day,$s_year); //f_sp = first startpoint
        $l_sp	= gmmktime($s_hour-$offset_hour,$s_min-$offset_min,0,$e_month,$e_day,$e_year); //l_sp = last startpoint

	//----- Check start and endtime -----
	If($s_time > $e_time) {
	    // Endtime is a part of next day
	    $f_ep 	= gmmktime($e_hour-$offset_hour,$e_min-$offset_min,0,$s_month,$s_day+1,$s_year); 
	    $l_ep	= gmmktime($e_hour-$offset_hour,$e_min-$offset_min,0,$e_month,$e_day+1,$e_year);
	}else {
	    // Endtime is a part of same day
	    $f_ep 	= gmmktime($e_hour-$offset_hour,$e_min-$offset_min,0,$s_month,$s_day,$s_year);
	    $l_ep	= gmmktime($e_hour-$offset_hour,$e_min-$offset_min,0,$e_month,$e_day,$e_year);
	}


	//----- ERROR CHECK (3) -----
	// Check whether start- and endpoint are part of future timestamps (Important for timespan "today")
	//$sql = "SELECT name_cache FROM data_template_data WHERE local_data_id = $local_data_id";
	//$rrd_name = db_fetch_cell($sql);

	if($f_sp > gmmktime()) {
	    run_error(3, $report_id, $local_data_id);
	    continue;
	}elseif($f_ep > gmmktime()) {
	    $f_ep	= gmmktime();
	    $l_ep	= gmmktime();
	    run_error(6, $report_id, $local_data_id);
	}elseif($l_ep > gmmktime()) {
	    run_error(6, $report_id, $local_data_id);
	}
	//---------------------------


	//----- Calculate shift duration -----
	$shift_duration = $f_ep - $f_sp;


	//----- Set options for rrd_fetch and run it! -----
	$rrd_data = array();
	
	switch($rrdtool_api) {
	    case '0':
	        //PHP bindings
        	$rrd_options 	= array( $rra_type, "--start", $f_sp, "--end", $l_ep);
    		$rrd_data	= rrd_fetch($data_source_path, $rrd_options, count($rrd_options));
		if ($error = rrd_error()){
		    run_error(5, $report_id, $local_data_id, $error);
		    continue;
		}
	        break;
		
	    case '1':
	        //RRDtool Cacti means the core function "rrdtool_execute"
		$cmd_line 	= "fetch $data_source_path $rra_type -s $f_sp -e $l_ep";
		$data 		= rrdtool_execute($cmd_line, false, RRDTOOL_OUTPUT_STDOUT);
                transform( $data, $rrd_data);
		break; 
		
	    case '2':
	        //RRDtool Server
 		
		$cmd_line	= "fetch $data_source_path $rra_type -s $f_sp -e $l_ep \n"; 
		socket_write($socket_handle, $cmd_line);
		$data		= '';
		while($out = socket_recv($socket_handle, $buffer, 16384, 0)) {
		    $data      .= $buffer;
		    if(strstr(substr($buffer, -30, 10), 'OK')) {
			$data	= substr($data, 0 , strpos($data, 'OK'));
			break;
                    }
                }
                transform( $data, $rrd_data);
	}


	//----- Read header informations from rrd_data array -----
        $rrd_f_mp 	= $rrd_data['start'] + $rrd_data['step'];	//rrd_f_mp = first measuring point
	$rrd_ep 	= $rrd_data['end'];
	$rrd_p_mp	= $rrd_data['end'] - $rrd_data['step'];		//rrd_p_mp = penultimate measuring point
	$rrd_step	= $rrd_data['step'];						
	$rrd_ds_cnt	= $rrd_data['ds_cnt'];
	$rrd_ds_namv	= $rrd_data['ds_namv'];

	//----- Generate all required informations for calculating ----- 
        $rrd_ad_data 	= &get_type_of_request($shift_startday, $shift_endday, $f_sp, $l_sp,
					       $shift_duration, $rrd_f_mp, $rrd_ep, $rrd_step,
					       $rrd_ds_cnt);

	//----- ERROR CHECK (5) -----
	// Check if startpoints are available 
	if($rrd_ad_data == false) {
	    run_error(7, $report_id, $local_data_id);
	    continue;
	}
	//---------------------------


        //----- Calculate correction factor for data source type: 'COUNTER' -----
	$corr_factor_start = ($rrd_f_mp - $f_sp)/$rrd_step;
	$corr_factor_end   = ($l_ep - $rrd_p_mp)/$rrd_step;

        //----- Analyse data source types and save id to "rrd_info" array -----
	// Waiting for Joey's solution with PHP Module ( for rrd_info() function )
        // => Using slow solution instead:
	for($j = 0; $j < $rrd_ds_cnt; $j++) {
    	    for($k = 0; $k < $rrd_tp_cnt; $k++) {
		if($data_source_types[$k]['data_source_name'] == $rrd_data['ds_namv'][$j]) {
		    $rrd_info['ds_type'][$j] = $data_source_types[$k]['data_source_type_id'];
		}
	    }
	}
        
        //----- Calculate interface availability -----



        //----- Prepare data for calculating -----
	$pre_data = &get_prepared_data($rrd_data['data'], $rrd_ad_data, $rrd_ds_cnt, 
				      $rrd_info, $corr_factor_start, $corr_factor_end);

	//----- ERROR CHECK (6) -----
	//Check if prepared data contains values after filtering NANs. 
	if(!count($pre_data[0]) > 0) {
	    run_error(8, $report_id, $local_data_id);
	    continue;
	}
	//---------------------------


	//----- Create calculating parameters -----
	$params['rrd_step']	  = $rrd_step;
	$params['rrd_ds_cnt']     = $rrd_ds_cnt;
	$params['variables']  	  = $report_definitions[3];
	$params['multiplier']	  = $report_definitions[4]['multiplier'];
	if(isset($bandwidth)) {
	    $params['bandwidth']  = $bandwidth;
	}

        //----- Start calculating function -----
	$results  = $calc_function($pre_data, $params);

	//----- Create new columns for table 'rrd_results_$report_id' -----
	if($create_result_table_columns == false) {
	    // Create the sql string
	    $list = '';
	    for( $j = 0; $j < $rrd_ds_cnt; $j++) {    
		foreach($results[$j] as $key => $value) {
		    $list = $list . " ADD `$rrd_ds_namv[$j]__$key` FLOAT,";
		}
	    }
	    // Remove last comma and complete the sql string
	    $list = substr($list, 0, strlen($list)-1);
	    $list = "ALTER TABLE reportit_results_$report_id $list";

	    // Add columms
	    db_execute($list);

	    // Update output variable
	    $create_result_table_columns = true;

	    // Update variable 'Datasource description'
	    foreach($rrd_ds_namv as $value) {
		$ds_description = $ds_description . "$value|";
	    }
	    // Remove last '|'
	    $ds_description = substr($ds_description, 0, strlen($ds_description)-1);

	    // Update variable 'Result description'
	    foreach($results[0] as $key => $value) {
		$result_description = $result_description . "$key|";
	    }
	    // Remove last '|'
	    $result_description = substr($result_description, 0, strlen($result_description)-1);

	    // Set report's state valid
	    $valid_report = true;
	}

	//----- Save results into MySQL database -----
	// Create the sql string
	$list = '';
	for( $j = 0; $j < $rrd_ds_cnt; $j++) {
	    foreach($results[$j] as $key => $value) {
		$list = $list . " `$rrd_ds_namv[$j]__$key` = '$value',";
	    }
	}
	// Remove last comma
	$list = substr($list, 0, strlen($list)-1);
	// Save values
	db_execute("REPLACE reportit_results_$report_id SET id = $local_data_id, $list");
    }

    //----- Close socket connection -----
    if($socket_handle != '' && !$run_scheduled) disc_rrdtool_server();
  
    //----- Make a note of our endpoint -----
    $runtime_ep = microtime();

    //----- Calculate runtime -----
    $runtime = get_runtime($runtime_sp, $runtime_ep);


    //----- ERROR CHECK (7) -----
    if($valid_report != true){
	run_error(4, $report_id);
	return $run_return;
    }
    //---------------------------


    //----- Save/update report data -----
    $now = date("Y-m-d H:i:s", gmmktime());
    
    db_execute("UPDATE reportit_reports 
		SET last_run 			= '$now',
		runtime 			= '$runtime',
		start_date 			= '$s_date',
		end_date 			= '$e_date',
		ds_description 			= '$ds_description',
		result_description 		= '$result_description'
		WHERE id 			= '$report_id'");

    //----- Return messages and runtime-----
    $run_return['runtime'] = $runtime;
    return $run_return;
}
?>